<?php
/**
 * ManiaLib - Lightweight PHP framework for Manialinks
 *
 * @see         http://code.google.com/p/manialib/
 * @copyright   Copyright (c) 2009-2011 NADEO (http://www.nadeo.com)
 * @license     http://www.gnu.org/licenses/lgpl.html LGPL License 3
 * @version     $Revision$:
 * @author      $Author$:
 * @date        $Date$:
 */

namespace ManiaLib\Utils;

/*
 * Info: pattern (?<!\$)((?:\$\$)*) match an even number of $
 */
abstract class Formatting
{
	/**
	 * Removes each single character code in $codes string
	 * Adding l, h or p will also strip links
	 * Adding an hexadecimal char will also strip colors
	 */
	static function stripCodes($string, $codes)
	{	
		$check = $codes;	//attempt to prevent var from being recast as bool in following tests. -cdv
		if(preg_match('/[hlp]/iu', $check))
			$string = self::stripLinks($string);
		if(preg_match('/[0-9a-f]/iu', $check))
			$string = self::stripColors($string);
		return preg_replace('/(?<!\$)((?:\$\$)*)\$['.$codes.']/iu', '$1', $string);
	}
	
	/**
	 * Removes wide, bold and shadowed
	 */
	static function stripWideFonts($string)
	{
		return self::stripCodes($string, 'wos');
	}

	/**
	 * Removes links
	 */
	static function stripLinks($string)
	{
		return preg_replace('/(?<!\$)((?:\$\$)*)\$[hlp](?:\[.*?\]|\[.*?$)?(.*?)(?:\$[hlp]|(\$z)|$)/iu', '$1$2$3', $string);
	}

	/**
	 * Removes colors
	 */
	static function stripColors($string)
	{
		return preg_replace('/(?<!\$)((?:\$\$)*)\$(?:g|[0-9a-f][^\$]{0,2})/iu', '$1', $string);
		       preg_replace('/(?<!\$)((?:\$\$)*)\$[ow<>]/iu', '$1', $string);
	}

	/**
	 * Removes all styles
	 */
	static function stripStyles($string)
	{
		$string = preg_replace('/(?<!\$)((?:\$\$)*)\$[^$0-9a-hlp]/iu', '$1', $string);
		$string = self::stripLinks($string);
		$string = self::stripColors($string);
		return $string;
	}
	// pasted this old reliable function in after giving up on above functions. -cdv
	/**
	* Summary: Strips all display formatting from an input string, suitable for display
	*          within the game ('$$' escape pairs are preserved) and for logging
	* Params : $input - The input string to strip formatting from
	*          $for_tm - Optional flag to double up '$' into '$$' (default, for TM) or not (for logs, etc)
	* Returns: The content portions of $input without formatting
	* Authors: Bilge/Assembler Maniac/Xymph/Slig
	*/
	static function stripFormatting($input, $for_tm = true) {
		return
			//Replace all occurrences of a null character back with a pair of dollar
			//signs for displaying in TM, or a single dollar for log messages etc.
			str_replace("\0", ($for_tm ? '$$' : '$'),
				//Replace links (introduced in TMU)
				preg_replace(
					'/
					#Strip TMF H, L & P links by stripping everything between each square
					#bracket pair until another $H, $L or $P sequence (or EoS) is found;
					#this allows a $H to close a $L and vice versa, as does the game
					\\$[hlp](.*?)(?:\\[.*?\\](.*?))*(?:\\$[hlp]|$)
					/ixu',
					//Keep the first and third capturing groups if present
					'$1$2',
					//Replace various patterns beginning with an unescaped dollar
					preg_replace(
						'/
						#Match a single dollar sign and any of the following:
						\\$
						(?:
							#Strip color codes by matching any hexadecimal character and
							#any other two characters following it (except $)
							[0-9a-f][^$][^$]
							#Strip any incomplete color codes by matching any hexadecimal
							#character followed by another character (except $)
							|[0-9a-f][^$]
							#Strip any single style code (including an invisible UTF8 char)
							#that is not an H, L or P link or a bracket ($[ and $])
							|[^][hlp]
							#Strip the dollar sign if it is followed by [ or ], but do not
							#strip the brackets themselves
							|(?=[][])
							#Strip the dollar sign if it is at the end of the string
							|$
						)
						#Ignore alphabet case, ignore whitespace in pattern & use UTF-8 mode
						/ixu',
						//Replace any matches with nothing (i.e. strip matches)
						'',
						//Replace all occurrences of dollar sign pairs with a null character
						str_replace('$$', "\0", $input)
					)
				)
			)
		;
	}
	
	static function contrastColors($string, $background)
	{
		$background = Color::StringToRgb24($background);
		return preg_replace_callback('/(?<!\$)((?:\$\$)*)(\$[0-9a-f][^\$]{0,2})/iu',
				function($matches) use ($background)
				{
					$color = Color::StringToRgb24($matches[2]);
					$color = Color::Contrast($color, $background);
					$color = Color::Rgb24ToRgb12($color);
					$color = Color::Rgb12ToString($color);
					return $matches[1].'$'.$color;
				}, $string);
	}
}

?>