<?php

namespace ManiaLivePlugins\eXpansion\AutoLoad\Gui\Windows;

/**
 * @author       Oliver de Cramer (oliverde8 at gmail.com)
 * @copyright    GNU GENERAL PUBLIC LICENSE
 *                     Version 3, 29 June 2007
 *
 * PHP version 5.3 and above
 *
 * LICENSE: This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see {http://www.gnu.org/licenses/}.
 */

use ManiaLive\Gui\Controls\Frame;
use ManiaLib\Gui\Elements\Label;
use ManiaLive\PluginHandler\PluginHandler;
use ManiaLivePlugins\eXpansion\AutoLoad\AutoLoad;
use ManiaLivePlugins\eXpansion\AutoLoad\Gui\Controls\Plugin;
use ManiaLivePlugins\eXpansion\Core\types\config\MetaData;
use ManiaLivePlugins\eXpansion\Faq\Gui\Controls\Line;
use ManiaLivePlugins\eXpansion\Gui\Elements\Button;
use ManiaLivePlugins\eXpansion\Gui\Elements\Dropdown;
use ManiaLivePlugins\eXpansion\Gui\Elements\Inputbox;
use ManiaLivePlugins\eXpansion\Gui\Windows\Window;
use ManiaLivePlugins\eXpansion\Gui\Elements\Pager;

class PluginList extends Window
{

    /**
     * @var Inputbox
     */
    protected $input_name;

    /** @var  Inputbox */
    protected $input_author;

    /** @var string /*
     * protected $value_name;
     *
     * /** @var  string
     */
    protected $value_author;

    /**
     * @var Dropdown
     */
    protected $select_group;

    /**
     * @var Label
     */
    protected $label_group;

    /**
     * @var String
     */
    protected $value_group;

    protected $elements = array();

    /**
     * @var Button
     */
    protected $button_search;

    /**
     * @var Pager
     */
    public $pagerFrame = null;

    /**
     * @var PluginHandler
     */
    protected $pluginHandler = null;

    /**
     * @var Plugin[]
     */
    protected $items = array();

    /**
     * @var MetaData[]
     */
    protected $pluginList = array();

    /**
     * @var AutoLoad
     */
    protected $autoLoad;
    /** @var Frame */
    protected $frame;

    /** @var bool */
    public $firstDisplay = true;

    /** @var Frame */
    public $categories;

    /** @var Button[] */
    protected $btn = [];

    protected function onConstruct()
    {
        parent::onConstruct();
        $login = $this->getRecipient();

        $layout = new \ManiaLib\Gui\Layouts\Line();
        $layout->setMarginWidth(2);
        $this->frame = new Frame(33, -8, $layout);

        $this->addComponent($this->frame);

        $layout = new \ManiaLib\Gui\Layouts\Column();
        $layout->setMarginHeight(0.5);
        $this->categories = new Frame(0, 0, $layout);
        $this->mainFrame->addComponent($this->categories);

        $this->input_name = new Inputbox('name');

        $this->input_name->setLabel(__("Name", $login));
        $this->frame->addComponent($this->input_name);

        $this->input_author = new Inputbox('author');
        $this->input_author->setLabel(__("Author", $login));
        $this->frame->addComponent($this->input_author);

        $this->select_group = new Dropdown("group", array('Select'), 0, 25);
        $this->frame->addComponent($this->select_group);

        $this->button_search = new Button();
        $this->button_search->setText(__("Search", $login));
        //$this->button_search->colorize('0a0');
        $this->button_search->setAction($this->createAction(array($this, "doSearch")));
        $this->frame->addComponent($this->button_search);

        $this->pagerFrame = new Pager();
        $this->pagerFrame->setPosition(33, -2);

        $this->mainFrame->addComponent($this->pagerFrame);

        $this->pluginHandler = PluginHandler::getInstance();

    }

    public function onResize($oldX, $oldY)
    {
        parent::onResize($oldX, $oldY);
        $this->pagerFrame->setSize($this->getSizeX() - 33, $this->getSizeY() - 4);
    }

    protected function onDraw()
    {
        parent::onDraw(); // TODO: Change the autogenerated stub
        foreach ($this->btn as $idx => $button) {
            if ($this->select_group->getSelected() == $idx) {
                $button->colorize("0b0");
            } else {
                $button->colorize(null);
            }
        }
    }


    /**
     * @param AutoLoad $autoLoader
     * @param MetaData[] $availablePlugins
     */
    public function populate(AutoLoad $autoLoader, $availablePlugins)
    {

        $this->pluginList = $availablePlugins;
        $this->autoLoad = $autoLoader;
        foreach ($this->items as $item) {
            $item->destroy();
        }
        $this->items = null;

        $this->pagerFrame->clearItems();

        $this->items = array();

        $groups = array();

        $i = 0;

        // $groups['All'] = true;

        foreach ($availablePlugins as $metaData) {
            if ($this->firstDisplay) {
                if (count($metaData->getGroups()) == 0) {
                    $groups["Other"] = true;
                }
                foreach ($metaData->getGroups() as $name) {
                    if ($name != "Core") {
                        $groups[$name] = true;
                    }
                }
            }
        }

        if ($this->firstDisplay) {
            $groups2 = array_keys($groups);
            sort($groups2, SORT_STRING);
            $this->elements = $groups2;
            $this->select_group->addItems($groups2);
            $this->select_group->setSelected(0);
            $this->value_group = $this->elements[0];

            foreach ($groups2 as $idx => $group) {
                $this->btn[$idx] = new Button();
                $this->btn[$idx]->setText($group);
                $this->btn[$idx]->setAction($this->createAction(array($this, "setGroup"), $idx));
                $this->categories->addComponent($this->btn[$idx]);
            }
        }

        /** @var MetaData[] $metaData */
        foreach ($availablePlugins as $metaData) {

            $text = $this->input_name->getText();
            if (!empty($text) && strpos(strtoupper($metaData->getName()), strtoupper($text)) === false) {
                continue;
            }

            $text = $this->input_author->getText();
            if (!empty($text) && strpos(strtoupper($metaData->getAuthor()), strtoupper($text)) === false) {
                continue;
            }

            if (!empty($this->value_group)
                && $this->value_group != "All"
                && !in_array($this->value_group, $metaData->getGroups())
            ) {
                continue;
            }

            // hide core plugins as you can't really load/unload them
            if (in_array("Core", $metaData->getGroups())) {
                continue;
            }

            $metaData->checkAll();
            $control = new Plugin(
                $i++,
                $autoLoader,
                $metaData,
                $this->getRecipient(),
                $this->pluginHandler->isLoaded($metaData->getPlugin())
            );
            $this->items[] = $control;
            $this->pagerFrame->addItem($control);
        }

        if ($this->firstDisplay) {
            $this->firstDisplay = false;
        }
    }

    public function destroy()
    {
        foreach ($this->items as $item) {
            $item->destroy();
        }
        foreach ($this->btn as $item) {
            $item->destroy();
        }
        $this->items = null;
        $this->btn = null;
        $this->pagerFrame->destroy();
        $this->destroyComponents();
        $this->autoLoad = null;

        parent::destroy();
    }

    public function setGroup($login, $groupIndex)
    {
        $this->value_group = $this->elements[$groupIndex];
        $this->populate($this->autoLoad, $this->pluginList);
        $this->select_group->setSelected($groupIndex);
        $this->redraw($login);
    }

    public function doSearch($login, $params)
    {
        $this->input_name->setText($params['name']);
        $this->input_author->setText($params['author']);
        $this->value_group = $params['group'] == "" ? 0 : $this->elements[$params['group']];

        $this->populate($this->autoLoad, $this->pluginList);
        $this->select_group->setSelected($params['group']);
        $this->redraw($login);
    }
}
